// python wrapper for vtkSIObject
//
#define VTK_WRAPPING_CXX
#define VTK_STREAMS_FWD_ONLY
#if !defined(__APPLE__)
#include "vtkPython.h"
#undef _XOPEN_SOURCE /* Conflicts with standards.h.  */
#undef _THREAD_SAFE /* Conflicts with pthread.h.  */
#endif
#include "vtkPythonArgs.h"
#include "vtkPythonOverload.h"
#include <vtksys/ios/sstream>
#include "vtkSIObject.h"

#if defined(WIN32)
extern "C" { __declspec( dllexport ) void PyVTKAddFile_vtkSIObject(PyObject *, const char *); }
#else
extern "C" { void PyVTKAddFile_vtkSIObject(PyObject *, const char *); }
#endif
#if defined(WIN32)
extern "C" { __declspec( dllexport ) PyObject *PyVTKClass_vtkSIObjectNew(const char *); }
#else
extern "C" { PyObject *PyVTKClass_vtkSIObjectNew(const char *); }
#endif

#ifndef DECLARED_PyVTKClass_vtkObjectNew
extern "C" { PyObject *PyVTKClass_vtkObjectNew(const char *); }
#define DECLARED_PyVTKClass_vtkObjectNew
#endif

static const char **PyvtkSIObject_Doc();


static PyObject *
PyvtkSIObject_GetClassName(PyObject *self, PyObject *args)
{
  vtkPythonArgs ap(self, args, "GetClassName");
  vtkObjectBase *vp = ap.GetSelfPointer(self, args);
  vtkSIObject *op = static_cast<vtkSIObject *>(vp);

  const char *tempr;
  PyObject *result = NULL;

  if (op && ap.CheckArgCount(0))
    {
    if (ap.IsBound())
      {
      tempr = op->GetClassName();
      }
    else
      {
      tempr = op->vtkSIObject::GetClassName();
      }

    if (!ap.ErrorOccurred())
      {
      result = ap.BuildValue(tempr);
      }
    }

  return result;
}


static PyObject *
PyvtkSIObject_IsA(PyObject *self, PyObject *args)
{
  vtkPythonArgs ap(self, args, "IsA");
  vtkObjectBase *vp = ap.GetSelfPointer(self, args);
  vtkSIObject *op = static_cast<vtkSIObject *>(vp);

  char *temp0 = NULL;
  int tempr;
  PyObject *result = NULL;

  if (op && ap.CheckArgCount(1) &&
      ap.GetValue(temp0))
    {
    if (ap.IsBound())
      {
      tempr = op->IsA(temp0);
      }
    else
      {
      tempr = op->vtkSIObject::IsA(temp0);
      }

    if (!ap.ErrorOccurred())
      {
      result = ap.BuildValue(tempr);
      }
    }

  return result;
}


static PyObject *
PyvtkSIObject_NewInstance(PyObject *self, PyObject *args)
{
  vtkPythonArgs ap(self, args, "NewInstance");
  vtkObjectBase *vp = ap.GetSelfPointer(self, args);
  vtkSIObject *op = static_cast<vtkSIObject *>(vp);

  vtkSIObject *tempr;
  PyObject *result = NULL;

  if (op && ap.CheckArgCount(0))
    {
    if (ap.IsBound())
      {
      tempr = op->NewInstance();
      }
    else
      {
      tempr = op->vtkSIObject::NewInstance();
      }

    if (!ap.ErrorOccurred())
      {
      result = ap.BuildVTKObject(tempr);
      if (result && PyVTKObject_Check(result))
        {
        PyVTKObject_GetObject(result)->UnRegister(0);
        PyVTKObject_SetFlag(result, VTK_PYTHON_IGNORE_UNREGISTER, 1);
        }
      }
    }

  return result;
}


static PyObject *
PyvtkSIObject_Initialize(PyObject *self, PyObject *args)
{
  vtkPythonArgs ap(self, args, "Initialize");
  vtkObjectBase *vp = ap.GetSelfPointer(self, args);
  vtkSIObject *op = static_cast<vtkSIObject *>(vp);

  vtkPVSessionCore *temp0 = NULL;
  PyObject *result = NULL;

  if (op && ap.CheckArgCount(1) &&
      ap.GetVTKObject(temp0, "vtkPVSessionCore"))
    {
    if (ap.IsBound())
      {
      op->Initialize(temp0);
      }
    else
      {
      op->vtkSIObject::Initialize(temp0);
      }

    if (!ap.ErrorOccurred())
      {
      result = ap.BuildNone();
      }
    }

  return result;
}

static PyMethodDef PyvtkSIObject_Methods[] = {
  {(char*)"GetClassName", PyvtkSIObject_GetClassName, 1,
   (char*)"V.GetClassName() -> string\nC++: const char *GetClassName()\n\n"},
  {(char*)"IsA", PyvtkSIObject_IsA, 1,
   (char*)"V.IsA(string) -> int\nC++: int IsA(const char *name)\n\n"},
  {(char*)"NewInstance", PyvtkSIObject_NewInstance, 1,
   (char*)"V.NewInstance() -> vtkSIObject\nC++: vtkSIObject *NewInstance()\n\n"},
  {(char*)"Initialize", PyvtkSIObject_Initialize, 1,
   (char*)"V.Initialize(vtkPVSessionCore)\nC++: virtual void Initialize(vtkPVSessionCore *session)\n\nInitializes the instance. Session is the session to which this\ninstance belongs to. During initialization, the SIObject\nbasically obtains ivars for necessary components.\n"},
  {NULL, NULL, 0, NULL}
};

PyObject *PyVTKClass_vtkSIObjectNew(const char *modulename)
{
  PyObject *cls = PyVTKClass_New(NULL,
    PyvtkSIObject_Methods,
    "vtkSIObject", modulename,
    NULL, NULL,
    PyvtkSIObject_Doc(),
    PyVTKClass_vtkObjectNew(modulename));
  return cls;
}

const char **PyvtkSIObject_Doc()
{
  static const char *docstring[] = {
    "vtkSIObject\n\n",
    "Superclass: vtkObject\n\n",
    "Object that is managed by vtkPVSessionCore which wrap concrete class\nsuch as the vtk ones.\n\n",
    NULL
  };

  return docstring;
}

void PyVTKAddFile_vtkSIObject(
  PyObject *dict, const char *modulename)
{
  PyObject *o;
  o = PyVTKClass_vtkSIObjectNew(modulename);

  if (o && PyDict_SetItemString(dict, (char *)"vtkSIObject", o) != 0)
    {
    Py_DECREF(o);
    }

}

