// python wrapper for vtkInitializationHelper
//
#define VTK_WRAPPING_CXX
#define VTK_STREAMS_FWD_ONLY
#if !defined(__APPLE__)
#include "vtkPython.h"
#undef _XOPEN_SOURCE /* Conflicts with standards.h.  */
#undef _THREAD_SAFE /* Conflicts with pthread.h.  */
#endif
#include "vtkPythonArgs.h"
#include "vtkPythonOverload.h"
#include <vtksys/ios/sstream>
#include "vtkInitializationHelper.h"

#if defined(WIN32)
extern "C" { __declspec( dllexport ) void PyVTKAddFile_vtkInitializationHelper(PyObject *, const char *); }
#else
extern "C" { void PyVTKAddFile_vtkInitializationHelper(PyObject *, const char *); }
#endif
#if defined(WIN32)
extern "C" { __declspec( dllexport ) PyObject *PyVTKClass_vtkInitializationHelperNew(const char *); }
#else
extern "C" { PyObject *PyVTKClass_vtkInitializationHelperNew(const char *); }
#endif

#ifndef DECLARED_PyVTKClass_vtkObjectNew
extern "C" { PyObject *PyVTKClass_vtkObjectNew(const char *); }
#define DECLARED_PyVTKClass_vtkObjectNew
#endif

static const char **PyvtkInitializationHelper_Doc();


static PyObject *
PyvtkInitializationHelper_GetClassName(PyObject *self, PyObject *args)
{
  vtkPythonArgs ap(self, args, "GetClassName");
  vtkObjectBase *vp = ap.GetSelfPointer(self, args);
  vtkInitializationHelper *op = static_cast<vtkInitializationHelper *>(vp);

  const char *tempr;
  PyObject *result = NULL;

  if (op && ap.CheckArgCount(0))
    {
    if (ap.IsBound())
      {
      tempr = op->GetClassName();
      }
    else
      {
      tempr = op->vtkInitializationHelper::GetClassName();
      }

    if (!ap.ErrorOccurred())
      {
      result = ap.BuildValue(tempr);
      }
    }

  return result;
}


static PyObject *
PyvtkInitializationHelper_IsA(PyObject *self, PyObject *args)
{
  vtkPythonArgs ap(self, args, "IsA");
  vtkObjectBase *vp = ap.GetSelfPointer(self, args);
  vtkInitializationHelper *op = static_cast<vtkInitializationHelper *>(vp);

  char *temp0 = NULL;
  int tempr;
  PyObject *result = NULL;

  if (op && ap.CheckArgCount(1) &&
      ap.GetValue(temp0))
    {
    if (ap.IsBound())
      {
      tempr = op->IsA(temp0);
      }
    else
      {
      tempr = op->vtkInitializationHelper::IsA(temp0);
      }

    if (!ap.ErrorOccurred())
      {
      result = ap.BuildValue(tempr);
      }
    }

  return result;
}


static PyObject *
PyvtkInitializationHelper_NewInstance(PyObject *self, PyObject *args)
{
  vtkPythonArgs ap(self, args, "NewInstance");
  vtkObjectBase *vp = ap.GetSelfPointer(self, args);
  vtkInitializationHelper *op = static_cast<vtkInitializationHelper *>(vp);

  vtkInitializationHelper *tempr;
  PyObject *result = NULL;

  if (op && ap.CheckArgCount(0))
    {
    if (ap.IsBound())
      {
      tempr = op->NewInstance();
      }
    else
      {
      tempr = op->vtkInitializationHelper::NewInstance();
      }

    if (!ap.ErrorOccurred())
      {
      result = ap.BuildVTKObject(tempr);
      if (result && PyVTKObject_Check(result))
        {
        PyVTKObject_GetObject(result)->UnRegister(0);
        PyVTKObject_SetFlag(result, VTK_PYTHON_IGNORE_UNREGISTER, 1);
        }
      }
    }

  return result;
}


static PyObject *
PyvtkInitializationHelper_Initialize_s1(PyObject *, PyObject *args)
{
  vtkPythonArgs ap(args, "Initialize");

  char *temp0 = NULL;
  int temp1;
  PyObject *result = NULL;

  if (ap.CheckArgCount(2) &&
      ap.GetValue(temp0) &&
      ap.GetValue(temp1))
    {
    vtkInitializationHelper::Initialize(temp0, temp1);

    if (!ap.ErrorOccurred())
      {
      result = ap.BuildNone();
      }
    }

  return result;
}

static PyObject *
PyvtkInitializationHelper_Initialize_s2(PyObject *, PyObject *args)
{
  vtkPythonArgs ap(args, "Initialize");

  char *temp0 = NULL;
  int temp1;
  vtkPVOptions *temp2 = NULL;
  PyObject *result = NULL;

  if (ap.CheckArgCount(3) &&
      ap.GetValue(temp0) &&
      ap.GetValue(temp1) &&
      ap.GetVTKObject(temp2, "vtkPVOptions"))
    {
    vtkInitializationHelper::Initialize(temp0, temp1, temp2);

    if (!ap.ErrorOccurred())
      {
      result = ap.BuildNone();
      }
    }

  return result;
}

static PyObject *
PyvtkInitializationHelper_Initialize(PyObject *self, PyObject *args)
{
  int nargs = vtkPythonArgs::GetArgCount(args);

  switch(nargs)
    {
    case 2:
      return PyvtkInitializationHelper_Initialize_s1(self, args);
    case 3:
      return PyvtkInitializationHelper_Initialize_s2(self, args);
    }

  vtkPythonArgs::ArgCountError(nargs, "Initialize");
  return NULL;
}



static PyObject *
PyvtkInitializationHelper_Finalize(PyObject *, PyObject *args)
{
  vtkPythonArgs ap(args, "Finalize");

  PyObject *result = NULL;

  if (ap.CheckArgCount(0))
    {
    vtkInitializationHelper::Finalize();

    if (!ap.ErrorOccurred())
      {
      result = ap.BuildNone();
      }
    }

  return result;
}

static PyMethodDef PyvtkInitializationHelper_Methods[] = {
  {(char*)"GetClassName", PyvtkInitializationHelper_GetClassName, 1,
   (char*)"V.GetClassName() -> string\nC++: const char *GetClassName()\n\n"},
  {(char*)"IsA", PyvtkInitializationHelper_IsA, 1,
   (char*)"V.IsA(string) -> int\nC++: int IsA(const char *name)\n\n"},
  {(char*)"NewInstance", PyvtkInitializationHelper_NewInstance, 1,
   (char*)"V.NewInstance() -> vtkInitializationHelper\nC++: vtkInitializationHelper *NewInstance()\n\n"},
  {(char*)"Initialize", PyvtkInitializationHelper_Initialize, 1,
   (char*)"V.Initialize(string, int)\nC++: static void Initialize(const char *executable, int type)\nV.Initialize(string, int, vtkPVOptions)\nC++: static void Initialize(const char *executable, int type,\n    vtkPVOptions *options)\n\nInitializes the server manager. Do not use the server manager\nbefore calling this.\n"},
  {(char*)"Finalize", PyvtkInitializationHelper_Finalize, 1,
   (char*)"V.Finalize()\nC++: static void Finalize()\n\nFinalizes the server manager. Do not use the server manager after\ncalling this.\n"},
  {NULL, NULL, 0, NULL}
};

PyObject *PyVTKClass_vtkInitializationHelperNew(const char *modulename)
{
  PyObject *cls = PyVTKClass_New(NULL,
    PyvtkInitializationHelper_Methods,
    "vtkInitializationHelper", modulename,
    NULL, NULL,
    PyvtkInitializationHelper_Doc(),
    PyVTKClass_vtkObjectNew(modulename));
  return cls;
}

const char **PyvtkInitializationHelper_Doc()
{
  static const char *docstring[] = {
    "vtkInitializationHelper - help class for python modules\n\n",
    "Superclass: vtkObject\n\n",
    "This class is used by the python modules when they are loaded from\npython (as opposed to pvpython). It simply initializes the server\nmanager so that it can be used.\n\n",
    NULL
  };

  return docstring;
}

void PyVTKAddFile_vtkInitializationHelper(
  PyObject *dict, const char *modulename)
{
  PyObject *o;
  o = PyVTKClass_vtkInitializationHelperNew(modulename);

  if (o && PyDict_SetItemString(dict, (char *)"vtkInitializationHelper", o) != 0)
    {
    Py_DECREF(o);
    }

}

