// python wrapper for vtkTableFFT
//
#define VTK_WRAPPING_CXX
#define VTK_STREAMS_FWD_ONLY
#if !defined(__APPLE__)
#include "vtkPython.h"
#undef _XOPEN_SOURCE /* Conflicts with standards.h.  */
#undef _THREAD_SAFE /* Conflicts with pthread.h.  */
#endif
#include "vtkPythonArgs.h"
#include "vtkPythonOverload.h"
#include <vtksys/ios/sstream>
#include "vtkTableFFT.h"

#if defined(WIN32)
extern "C" { __declspec( dllexport ) void PyVTKAddFile_vtkTableFFT(PyObject *, const char *); }
#else
extern "C" { void PyVTKAddFile_vtkTableFFT(PyObject *, const char *); }
#endif
#if defined(WIN32)
extern "C" { __declspec( dllexport ) PyObject *PyVTKClass_vtkTableFFTNew(const char *); }
#else
extern "C" { PyObject *PyVTKClass_vtkTableFFTNew(const char *); }
#endif

#ifndef DECLARED_PyVTKClass_vtkTableAlgorithmNew
extern "C" { PyObject *PyVTKClass_vtkTableAlgorithmNew(const char *); }
#define DECLARED_PyVTKClass_vtkTableAlgorithmNew
#endif

static const char **PyvtkTableFFT_Doc();


static PyObject *
PyvtkTableFFT_GetClassName(PyObject *self, PyObject *args)
{
  vtkPythonArgs ap(self, args, "GetClassName");
  vtkObjectBase *vp = ap.GetSelfPointer(self, args);
  vtkTableFFT *op = static_cast<vtkTableFFT *>(vp);

  const char *tempr;
  PyObject *result = NULL;

  if (op && ap.CheckArgCount(0))
    {
    if (ap.IsBound())
      {
      tempr = op->GetClassName();
      }
    else
      {
      tempr = op->vtkTableFFT::GetClassName();
      }

    if (!ap.ErrorOccurred())
      {
      result = ap.BuildValue(tempr);
      }
    }

  return result;
}


static PyObject *
PyvtkTableFFT_IsA(PyObject *self, PyObject *args)
{
  vtkPythonArgs ap(self, args, "IsA");
  vtkObjectBase *vp = ap.GetSelfPointer(self, args);
  vtkTableFFT *op = static_cast<vtkTableFFT *>(vp);

  char *temp0 = NULL;
  int tempr;
  PyObject *result = NULL;

  if (op && ap.CheckArgCount(1) &&
      ap.GetValue(temp0))
    {
    if (ap.IsBound())
      {
      tempr = op->IsA(temp0);
      }
    else
      {
      tempr = op->vtkTableFFT::IsA(temp0);
      }

    if (!ap.ErrorOccurred())
      {
      result = ap.BuildValue(tempr);
      }
    }

  return result;
}


static PyObject *
PyvtkTableFFT_NewInstance(PyObject *self, PyObject *args)
{
  vtkPythonArgs ap(self, args, "NewInstance");
  vtkObjectBase *vp = ap.GetSelfPointer(self, args);
  vtkTableFFT *op = static_cast<vtkTableFFT *>(vp);

  vtkTableFFT *tempr;
  PyObject *result = NULL;

  if (op && ap.CheckArgCount(0))
    {
    if (ap.IsBound())
      {
      tempr = op->NewInstance();
      }
    else
      {
      tempr = op->vtkTableFFT::NewInstance();
      }

    if (!ap.ErrorOccurred())
      {
      result = ap.BuildVTKObject(tempr);
      if (result && PyVTKObject_Check(result))
        {
        PyVTKObject_GetObject(result)->UnRegister(0);
        PyVTKObject_SetFlag(result, VTK_PYTHON_IGNORE_UNREGISTER, 1);
        }
      }
    }

  return result;
}


static PyObject *
PyvtkTableFFT_SafeDownCast(PyObject *, PyObject *args)
{
  vtkPythonArgs ap(args, "SafeDownCast");

  vtkObject *temp0 = NULL;
  vtkTableFFT *tempr;
  PyObject *result = NULL;

  if (ap.CheckArgCount(1) &&
      ap.GetVTKObject(temp0, "vtkObject"))
    {
    tempr = vtkTableFFT::SafeDownCast(temp0);

    if (!ap.ErrorOccurred())
      {
      result = ap.BuildVTKObject(tempr);
      }
    }

  return result;
}

static PyMethodDef PyvtkTableFFT_Methods[] = {
  {(char*)"GetClassName", PyvtkTableFFT_GetClassName, 1,
   (char*)"V.GetClassName() -> string\nC++: const char *GetClassName()\n\n"},
  {(char*)"IsA", PyvtkTableFFT_IsA, 1,
   (char*)"V.IsA(string) -> int\nC++: int IsA(const char *name)\n\n"},
  {(char*)"NewInstance", PyvtkTableFFT_NewInstance, 1,
   (char*)"V.NewInstance() -> vtkTableFFT\nC++: vtkTableFFT *NewInstance()\n\n"},
  {(char*)"SafeDownCast", PyvtkTableFFT_SafeDownCast, 1,
   (char*)"V.SafeDownCast(vtkObject) -> vtkTableFFT\nC++: vtkTableFFT *SafeDownCast(vtkObject* o)\n\n"},
  {NULL, NULL, 0, NULL}
};

static vtkObjectBase *PyvtkTableFFT_StaticNew()
{
  return vtkTableFFT::New();
}

PyObject *PyVTKClass_vtkTableFFTNew(const char *modulename)
{
  PyObject *cls = PyVTKClass_New(&PyvtkTableFFT_StaticNew,
    PyvtkTableFFT_Methods,
    "vtkTableFFT", modulename,
    NULL, NULL,
    PyvtkTableFFT_Doc(),
    PyVTKClass_vtkTableAlgorithmNew(modulename));
  return cls;
}

const char **PyvtkTableFFT_Doc()
{
  static const char *docstring[] = {
    "vtkTableFFT - FFT for table columns\n\n",
    "Superclass: vtkTableAlgorithm\n\n",
    "vtkTableFFT performs the Fast Fourier Transform on the columns of a\ntable. Internally, it shoves each column into an image data and then\nuses vtkImageFFT to perform the actual FFT.\n\nSee Also:\n\nvtkImageFFT\n\n",
    NULL
  };

  return docstring;
}

void PyVTKAddFile_vtkTableFFT(
  PyObject *dict, const char *modulename)
{
  PyObject *o;
  o = PyVTKClass_vtkTableFFTNew(modulename);

  if (o && PyDict_SetItemString(dict, (char *)"vtkTableFFT", o) != 0)
    {
    Py_DECREF(o);
    }

}

