// Loadable modules
//
// Generated by ../../bin/kwProcessXML
//
#ifndef __module_blotish__h
#define __module_blotish__h

#include <string.h>


// From file /home/users/builder/rpm/BUILD/ParaView-3.12.0/Plugins/pvblot/blotish.py
static const char* module_blotish_blotish_string0 =
"\"\"\"\n"
"\n"
"The blotish module provides a set of commands that control batch ParaView\n"
"visualization using semantics similar to the blot program.\n"
"\n"
"Instead of calling the public methods of blotish directly this module is\n"
"driven by pvblot.  The pvblot module creates an interpreter that translates\n"
"blot syntax to botish method calls.  See pvblot.\n"
"\n"
"\"\"\"\n"
"\n"
"#==============================================================================\n"
"#\n"
"#  Program:   ParaView\n"
"#  Module:    blotish.py\n"
"#\n"
"#  Copyright (c) Kitware, Inc.\n"
"#  All rights reserved.\n"
"#  See Copyright.txt or http://www.paraview.org/HTML/Copyright.html for details.\n"
"#\n"
"#     This software is distributed WITHOUT ANY WARRANTY; without even\n"
"#     the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR\n"
"#     PURPOSE.  See the above copyright notice for more information.\n"
"#\n"
"#==============================================================================\n"
"\n"
"#-------------------------------------------------------------------------\n"
"# Copyright 2009 Sandia Corporation.\n"
"# Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,\n"
"# the U.S. Government retains certain rights in this software.\n"
"#-------------------------------------------------------------------------\n"
"\n"
"import paraview.simple\n"
"import async_io_helper\n"
"import timestep_selection\n"
"import number_list_parser\n"
"import blot_common\n"
"import tplot as tplot_mod\n"
"import math\n"
"import re\n"
"\n"
"# Global variable state is used by methods in this module\n"
"# After _init_blotish is called state is an instance of\n"
"# the class _State.\n"
"state = None\n"
"\n"
"# The exception class that is raised by this module\n"
"BlotishError = blot_common.Error\n"
"\n"
"# Some constants\n"
"NODE_VARIABLE = blot_common.NODE_VARIABLE\n"
"ELEMENT_VARIABLE = blot_common.ELEMENT_VARIABLE\n"
"GLOBAL_VARIABLE = blot_common.GLOBAL_VARIABLE\n"
"EXODUS = blot_common.EXODUS\n"
"SPYPLOT = blot_common.SPYPLOT\n"
"\n"
"STANDARD_COLORS = blot_common.STANDARD_COLORS\n"
"\n"
"\n"
"def _subprogram(names=[]):\n"
"    \"\"\"A decorator factor that can be applied to methods to signal that the\n"
"    method requires a specifc subprogram or subprograms.  The argument to this\n"
"    factor method can be a single string to require a specific subpgram,\n"
"    a list of strings to require one of a set of subprograms, or if no argument\n"
"    is provided than the method is valid if any subprogram is active.\"\"\"\n"
"    def decorator_func(func):\n"
"        if isinstance(names, str): programs = [names]\n"
"        else: programs = list(names)\n"
"        def wrapper(*args, **kwargs):\n"
"            if not state.subProgram \\\n"
"                or (state.subProgram and programs\n"
"                    and state.subProgram not in programs):\n"
"                if programs:\n"
"                    if len(programs) > 1: plural = \"s\"\n"
"                    else: plural = \"\"\n"
"                    msg = \"This command is only available in the subprogram%s: %s\" % (plural,\n"
"                                                                         \", \".join(programs))\n"
"                else:\n"
"                    msg = \"Expected a subprogram name\"\n"
"                raise BlotishError(msg)\n"
"            func(*args, **kwargs)\n"
"        wrapper.__name__ = func.__name__\n"
"        wrapper.__doc__ = func.__doc__\n"
"        return wrapper\n"
"    return decorator_func\n"
"\n"
"\n"
"class _State(object):\n"
"    \"This class holds state variables for blotish including PV objects.\"\n"
"\n"
"    def __init__(self):\n"
"        self.filename = None\n"
"        self.numBlocks = 1\n"
"        self.numBlockColors = len(STANDARD_COLORS)\n"
"        self.numSpectrumColors = 5\n"
"        self.filename_counter = 0\n"
"        self.mlines = 1\n"
"        self.msurface = 1\n"
"        self.tplot = None\n"
"        self.autoplot = True\n"
"        self.subProgram = None\n"
"        self.ioHelper = async_io_helper.new_helper()\n"
"        self.interactive = True\n"
"        self.diskwrite = False\n"
"        self.time_selection = None # will be set to an instance of timestep_selection\n"
"\n"
"        # set some default values acceptable for the ExodusIIReader\n"
"        # these may be adjusted for the SpyPlotReader\n"
"        self._block_id_variable = \"ObjectId\"\n"
"        self._block_id_field = \"CELL_DATA\"\n"
"        self._block_id_offset = 1\n"
"\n"
"    def __del__(self):\n"
"\n"
"        # Remove tplot first\n"
"        del self.tplot\n"
"\n"
"        # Sometimes paraview is already unloaded at this point.\n"
"        if not paraview: return\n"
"\n"
"        # Remove pipeline objects\n"
"        for name in self._pipelineObjects.keys():\n"
"            source = self._pipelineObjects.pop(name)\n"
"            paraview.simple.Delete(source)\n"
"\n"
"        # Finally remove the reader\n"
"        paraview.simple.Delete(self.reader)\n"
"\n"
"    def _getPrompt(self):\n"
"        if self.subProgram == \"detour\": return \"DETOUR> \"\n"
"        if self.subProgram == \"tplot\": return \"TPLOT> \"\n"
"        if self.subProgram == \"splot\": return \"SPLOT> \"\n"
"        else: return \"BLOT> \"\n"
"    prompt = property(_getPrompt, doc=\"Return the prompt for the active subprogram.\")\n"
"\n"
"    _pipelineObjects = {}\n"
"    def _getPipelineObject(self, name, createFunc, **newObjectOptions):\n"
"        if not self._pipelineObjects.has_key(name):\n"
"            self._pipelineObjects[name] = createFunc(**newObjectOptions)\n"
"            paraview.simple.Hide(self._pipelineObjects[name])\n"
"        return self._pipelineObjects[name]\n"
"\n"
"    def hasPipelineObject(self, name):\n"
"        return self._pipelineObjects.has_key(name)\n"
"\n"
"    _reader = None\n"
"    def _getReader(self):\n"
"        if not self.filename:\n"
"            raise BlotishError(\"No filename specified.\")\n"
"        if not self._reader:\n"
"            # matches patterns that end with \"spcth\" or spcth.<num>\n"
"            if paraview.servermanager.vtkSMReaderFactory.CanReadFile(\\\n"
"              self.filename, \"sources\", \"spcthreader\",\\\n"
"              paraview.servermanager.ActiveConnection.ID):\n"
"                # For the spy plot reader we will turn on all variables\n"
"                # and then apply the Extract CTH Parts filter to give us\n"
"                # something to work with\n"
"                r = paraview.simple.SpyPlotReader(FileName=self.filename)\n"
"                r.CellArrays = r.CellArrays.Available\n"
"                extract = paraview.simple.ExtractCTHParts()\n"
"                material_vars = []\n"
"                for var_name in r.CellArrays:\n"
"                    if var_name.startswith(\"Material volume fraction\"):\n"
"                        material_vars.append(var_name)\n"
"\n"
"                extract.UnsignedCharacterVolumeArrays = material_vars\n"
"\n"
"                extract.add_attribute(\"DatabaseType\", SPYPLOT)\n"
"                extract.add_attribute(\"FileName\", r.FileName)\n"
"                extract.add_attribute(\"TimestepValues\", r.TimestepValues)\n"
"\n"
"                self._reader = extract\n"
"                self._block_id_variable = \"Part Index\"\n"
"                self._block_id_field = \"POINT_DATA\"\n"
"                self._block_id_offset = 0\n"
"                \n"
"            else:\n"
"                self._reader = paraview.simple.ExodusIIReader(FileName=self.filename)\n"
"                self._reader.add_attribute(\"DatabaseType\", EXODUS)\n"
"\n"
"        return self._reader\n"
"\n"
"    reader = property(_getReader, doc=\"ParaView Exodus reader pipeline object.\")\n"
"\n"
"    _surface = None\n"
"    def _getSurface(self):\n"
"        return self._getPipelineObject('surface',\n"
"                                       paraview.simple.ExtractSurface,\n"
"                                       Input=self.reader)\n"
"    surface = property(_getSurface,\n"
"                       doc=\"ParaView surface extract pipeline object.\")\n"
"\n"
"    _glyph = None\n"
"    def _getGlyph(self):\n"
"        return self._getPipelineObject('glyph', paraview.simple.Glyph,\n"
"                                       Input=self.surface)\n"
"    glyph = property(_getGlyph, doc=\"ParaView glyph pipeline object.\")\n"
"\n"
"    _contour = None\n"
"    def _getContour(self):\n"
"        return self._getPipelineObject('contour', paraview.simple.Contour,\n"
"                                       Input=self.reader)\n"
"    contour = property(_getContour, doc=\"ParaView contour pipeline object.\")\n"
"\n"
"    def resetPipeline(self):\n"
"        for obj in self._pipelineObjects.values():\n"
"            if obj:\n"
"                paraview.simple.Hide(obj)\n"
"                paraview.simple.SetDisplayProperties(obj, Opacity=1)\n"
"\n"
"    # The last variable selected.\n"
"    currentVariable = None\n"
"    # The type of the last variable selected (NODE_VARIABLE, ELEMENT_VARIABLE, GLOBAL_VARIABLE)\n"
"    currentVariableType = None\n"
"\n"
"    def loadVariable(self, name):\n"
"        \"\"\"\n"
"        Finds the variable with name (case insensitive) and makes sure that\n"
"        the variable is loaded in the reader.  If that variable exists (and\n"
"        is loaded), then the currentVariable and currentVariableType fields\n"
"        of this object are set.  If the variable is not found, an exception is\n"
"        raised.\n"
"        \"\"\"\n"
"        var = _find_variable_or_raise_exception(self.reader, name)\n"
"        self.currentVariable = var.name\n"
"        self.currentVariableType = var.type\n"
"        self.currentVariableComponent = var.component\n"
"\n"
"    def get_next_screenshot_filename(self, label=\"\"):\n"
"        self.filename_counter += 1\n"
"        return \"pvblot_%05d_%s.png\" % (self.filename_counter, label)\n"
"\n"
"    def _getCurrentVariableInfo(self):\n"
"        if not self.currentVariable:\n"
"            raise BlotishError(\"No variable selected.\")\n"
"        r = self.reader\n"
"        itr = zip([NODE_VARIABLE, ELEMENT_VARIABLE, GLOBAL_VARIABLE],\n"
"                  [r.PointData, r.CellData, r.FieldData])\n"
"        for var_type, var_data in itr:\n"
"            if self.currentVariableType == var_type:\n"
"                return var_data[self.currentVariable]\n"
"        return None\n"
"    currentVariableInfo = property(_getCurrentVariableInfo)\n"
"\n"
"    # The handling of the lookup table may change dramatically as\n"
"    # paraview.simple is improved.\n"
"    _blockLookupTable = None\n"
"    def _getBlockLookupTable(self):\n"
"        if not self._blockLookupTable:\n"
"            self._blockLookupTable = paraview.servermanager.rendering.PVLookupTable()\n"
"            paraview.servermanager.Register(self._blockLookupTable)\n"
"            self._blockLookupTable.ColorSpace = \"RGB\"\n"
"        lt = self._blockLookupTable\n"
"        if len(lt.RGBPoints)/4 != self.numBlocks:\n"
"            self._rebuildBlockLookupTable(lt)\n"
"        return lt\n"
"    blockLookupTable = property(_getBlockLookupTable,\n"
"                                doc=\"Lookup table used for coloring blocks.\")\n"
"\n"
"    def rebuildBlockLookupTable(self):\n"
"        self._rebuildBlockLookupTable(self.blockLookupTable)\n"
"    def _rebuildBlockLookupTable(self, lt):\n"
"        # Add RGB points to lookup table.  These are tuples of 4 values.  First\n"
"        # one is the scalar values, the other 3 the RGB values.\n"
"        rgbpoints = []\n"
"        for i in xrange(self.numBlocks):\n"
"            rgbpoints.append(i + self._block_id_offset)\n"
"            rgbpoints.extend(STANDARD_COLORS[i % self.numBlockColors])\n"
"        lt.RGBPoints = rgbpoints\n"
"\n"
"    _spectrumLookupTable = None\n"
"    _spectrumLookupTableVariable = ''\n"
"    def _getSpectrumLookupTable(self):\n"
"        if not self._spectrumLookupTable:\n"
"            self._spectrumLookupTable = paraview.servermanager.rendering.PVLookupTable()\n"
"            paraview.servermanager.Register(self._spectrumLookupTable)\n"
"            self._spectrumLookupTable.ColorSpace = \"HSV\"\n"
"        lt = self._spectrumLookupTable\n"
"        if len(lt.RGBPoints)/8 != self.numSpectrumColors+1 or \\\n"
"           self._spectrumLookupTableVariable != self.currentVariable:\n"
"            self._rebuildSpectrumLookupTable(lt)\n"
"        return lt\n"
"    spectrumLookupTable = property(_getSpectrumLookupTable,\n"
"                                   doc=\"Lookup table used for contours and painting.\")\n"
"\n"
"    def rebuildSpectrumLookupTable(self):\n"
"        self._rebuildSpectrumLookupTable(self.spectrumLookupTable)\n"
"    def _rebuildSpectrumLookupTable(self, lt):\n"
"        import paraview.vtk\n"
"        # Find a good range for the data.\n"
"        range = self.currentVariableInfo.GetRange(self.currentVariableComponent)\n"
"        \n"
"        if self.currentVariableComponent == -1:\n"
"            lt.VectorMode = \"Magnitude\"\n"
"        else:\n"
"            lt.VectorMode = \"Component\"\n"
"            lt.VectorComponent = self.currentVariableComponent\n"
"\n"
"        hueStep = (2.0/3.0)/(self.numSpectrumColors)\n"
"        rangeStep = (range[1]-range[0])/(self.numSpectrumColors+1)\n"
"        # Add RGB points to lookup table.  These are tuples of 4 values.  First\n"
"        # one is the scalar values, the other 3 the RGB values.\n"
"        rgbpoints = []\n"
"        for i in xrange(self.numSpectrumColors+1):\n"
"            # Add two colors for a constant range.\n"
"            color = paraview.vtk.vtkMath.HSVToRGB(2.0/3.0 - i*hueStep, 1, 1)\n"
"            rgbpoints.append(range[0]+i*rangeStep)\n"
"            rgbpoints.extend(color)\n"
"            rgbpoints.append(range[0]+i*rangeStep+0.9999*rangeStep)\n"
"            rgbpoints.extend(color)\n"
"        lt.RGBPoints = rgbpoints\n"
"\n"
"    def _getSpectrumScalarValues(self):\n"
"        return self.spectrumLookupTable.RGBPoints[8::8]\n"
"    spectrumScalarValues = property(_getSpectrumScalarValues,\n"
"                                    doc=\"The scalar values associated with each spectrum contour.\")\n"
"\n"
"    def getPlotTimes(self):\n"
"        return self.time_selection.get_selected_times()\n"
"\n"
"\n"
"def _cleanup():\n"
"    global state\n"
"    del state\n"
"    state = None\n"
"\n"
"def _get_prompt():\n"
"    if state.ioHelper.get_output():\n"
"        return str(state.ioHelper.get_output())\n"
"    return state.prompt\n"
"\n"
"def _handle_input(line):\n"
"    return state.ioHelper.handle_input(line)\n"
"\n"
"def _get_io_helper():\n"
"    return state.ioHelper\n"
"\n"
"def _set_interactive(value):\n"
"    \"\"\"Sets interactive mode on/off.  Sets diskwrite mode to the opposite value.\"\"\"\n"
"    state.interactive = value\n"
"    state.diskwrite = not value\n"
"\n"
"def _find_variable_or_raise_exception(reader, name):\n"
"    \"\"\"Returns a valid blot_common.Variable instance or raises a BlotishError.\"\"\"\n"
"    var = blot_common.find_variable(reader, name)\n"
"    if not var:\n"
"        raise BlotishError(\"'%s' is an invalid variable name\" % name)\n"
"    return var\n"
"\n"
"def _init_blotish(filename):\n"
"    \"\"\"\n"
"    This command initializes the blotish commands.  Think of this as the\n"
"    super-command that must be run before all other sub-commands.\n"
"\n"
"    This function takes a single string argument that is the path to the\n"
"    exodus file to load.\n"
"    \"\"\"\n"
"    global state\n"
"    if state: return\n"
"    elif not filename: raise BlotishError(\"You need to specify a filename.\")\n"
"\n"
"    state = _State()\n"
"    state.filename = filename\n"
"\n"
"    # Make sure a render window has been created\n"
"    state.renderview = paraview.simple.GetRenderView()\n"
"    state.renderview.UseOffscreenRenderingForScreenshots = 0\n"
;

// From file /home/users/builder/rpm/BUILD/ParaView-3.12.0/Plugins/pvblot/blotish.py
static const char* module_blotish_blotish_string1 =
"\n"
"    # This will automatically load the reader, filename must be set first.\n"
"    reader = state.reader\n"
"\n"
"    if not reader:\n"
"        state.filename = None\n"
"        raise BlotishError(\"Failed to load file %s\" % filename)\n"
"\n"
"    reader.UpdatePipeline()\n"
"    datainfo = reader.GetDataInformation()\n"
"    state.numBlocks = datainfo.GetNumberOfDataSets()\n"
"\n"
"    print \"Database:\", reader.FileName\n"
"    print\n"
"    print \"Number of nodes          =\", datainfo.GetNumberOfPoints()\n"
"    print \"Number of elements       =\", datainfo.GetNumberOfCells()\n"
"    print \"Number of element blocks =\", state.numBlocks\n"
"    print\n"
"\n"
"    if reader.DatabaseType == EXODUS:\n"
"        print \"Number of node sets      =\", len(reader.NodeSetArrayStatus.Available)\n"
"        print \"Number of side sets      =\", len(reader.SideSetArrayStatus.Available)\n"
"        print\n"
"        print \"Variable Names\"\n"
"        print \"Global: \",\n"
"        for name in reader.GlobalVariables.Available:\n"
"            print \" \", name,\n"
"        print\n"
"        print \"Nodal:  \",\n"
"        for name in reader.PointVariables.Available:\n"
"            print \" \", name,\n"
"        print\n"
"        print \"Element:\",\n"
"        for name in reader.ElementVariables.Available:\n"
"            print \" \", name,\n"
"\n"
"    elif reader.DatabaseType == SPYPLOT:\n"
"\n"
"        print \"Variable Names\"\n"
"        print \"Element:\",\n"
"        for name in reader.Input.CellArrays.Available:\n"
"            print \" \", name,\n"
"\n"
"    print\n"
"    print \"Number of time steps     =\", len(reader.TimestepValues)\n"
"    if len(reader.TimestepValues) > 0:\n"
"        print \"   Minimum time          =\", min(reader.TimestepValues)\n"
"        print \"   Maximum time          =\", max(reader.TimestepValues)\n"
"        all_times = reader.TimestepValues\n"
"    else:\n"
"        all_times = [0]\n"
"    print\n"
"    state.time_selection = timestep_selection.TimestepSelection(all_times)\n"
"\n"
"def _finish_plot_change():\n"
"    \"\"\"Maybe re-render depending on the value of state.autoplot\"\"\"\n"
"    if state.autoplot:\n"
"        paraview.simple.Render()\n"
"\n"
"def _find_variable_command(variable_name):\n"
"    \"\"\"When a command is entered that doesn't match any existing command it\n"
"    might be a variable name in which case we should call typlot with that\n"
"    variable as the first argument.\"\"\"\n"
"    def call_typlot(*args, **kwargs):\n"
"        args = list(args)\n"
"        args.insert(0, variable_name)\n"
"        typlot(*args, **kwargs)\n"
"    call_typlot.__name__ = typlot.__name__\n"
"    call_typlot.__doc__ = typlot.__doc__\n"
"\n"
"    var = blot_common.find_variable(state.reader, variable_name)\n"
"    if var: return call_typlot\n"
"    return None\n"
"\n"
"@_subprogram(\"tplot\")\n"
"@async_io_helper.wrap(_get_io_helper)\n"
"def typlot(*args, **kwargs):\n"
"    \"\"\"Create a time plot of the given variable\"\"\"\n"
"\n"
"    _require_plot_mode(tplot_mod.TYCURVE)\n"
"\n"
"    # If args not specified then get args using asynchronous input\n"
"    if not args:\n"
"        yield \"Y VARIABLE \"\n"
"        args = kwargs[\"io_helper\"].get_input().split()\n"
"        if not args: args = [\" \"]\n"
"\n"
"    args = list(args)\n"
"    var = _find_variable_or_raise_exception(state.reader, args.pop(0))\n"
"\n"
"    if var.type == GLOBAL_VARIABLE:\n"
"        if args:\n"
"            blot_common.print_blot_warning(\"Extra tokens after global \"\n"
"                                           \"variable name ignored.\")\n"
"        c = tplot_mod.Curve(var)\n"
"        state.tplot.add_curve(c)\n"
"        state.tplot.print_show()\n"
"        return\n"
"\n"
"    if not args:\n"
"        raise BlotishError(\"Expected node/element number\")\n"
"\n"
"    s = \" \".join(args)\n"
"    try: selected_ids = number_list_parser.parse_number_list(s, int)\n"
"    except number_list_parser.Error, err: raise BlotishError(err)\n"
"\n"
"    uniq = dict()\n"
"    selected_ids = [uniq.setdefault(i,i) for i in selected_ids if i not in uniq]\n"
"\n"
"    for id in selected_ids:\n"
"        c = tplot_mod.Curve(var, id)\n"
"        state.tplot.add_curve(c)\n"
"    state.tplot.print_show()\n"
"\n"
"\n"
"def _require_plot_mode(mode):\n"
"    \"\"\"Check if the current tplot mode is the given mode.  If there is a mismatch\n"
"    raise a BlotishError.\"\"\"\n"
"    current_mode = state.tplot.get_current_curve_mode()\n"
"    if current_mode is not None:\n"
"        if mode != current_mode:\n"
"            raise BlotishError(\"Time curves and X-Y curves must be defined separately\")\n"
"\n"
"\n"
"@_subprogram(\"tplot\")\n"
"@async_io_helper.wrap(_get_io_helper)\n"
"def xyplot(io_helper=None):\n"
"    \"\"\"Create a variable versus variable plot with data points at each timestep\"\"\"\n"
"\n"
"    _require_plot_mode(tplot_mod.XYCURVE)\n"
"\n"
"    def extract_variable_and_ids(args):\n"
"        # convert to list of at least one token\n"
"        if not args: args = [\" \"]\n"
"        args = list(args)\n"
"\n"
"        # treat the first token as a variable name to lookup a variable\n"
"        var = _find_variable_or_raise_exception(state.reader, args.pop(0))\n"
"\n"
"        # variable must not be a global\n"
"        if var.type == GLOBAL_VARIABLE:\n"
"            raise BlotishError(\"XYPlot does not support global variables\")\n"
"            return\n"
"\n"
"        # Now parse the remaining tokens as a list of numbers and number ranges\n"
"        if not args: raise BlotishError(\"Expected node/element number\")\n"
"        s = \" \".join(args)\n"
"        try: selected_ids = number_list_parser.parse_number_list(s, int)\n"
"        except number_list_parser.Error, err: raise BlotishError(err)\n"
"\n"
"        # Convert to list of unique values and return\n"
"        uniq = dict()\n"
"        selected_ids = [uniq.setdefault(i,i) for i in selected_ids if i not in uniq]\n"
"        return var, selected_ids\n"
"\n"
"    yield \"X VARIABLE \"\n"
"    args = io_helper.get_input().split()\n"
"    x_var, x_ids = extract_variable_and_ids(args)\n"
"\n"
"    yield \"Y VARIABLE \"\n"
"    args = io_helper.get_input().split()\n"
"    y_var, y_ids = extract_variable_and_ids(args)\n"
"\n"
"    if len(x_ids) != len(y_ids):\n"
"        raise BlotishError(\"The number of selected node or element ids must be \"\n"
"                           \"the same for the X variable and Y variable.\")\n"
"\n"
"    if x_var.type != y_var.type:\n"
"        raise BlotishError(\"X and Y variable must be the same type (node or element)\")\n"
"\n"
"    for xid, yid in zip(x_ids, y_ids):\n"
"        if xid != yid:\n"
"            raise BlotishError(\"XYPlot does not support plotting variables at \"\n"
"                               \"different node or element ids against each other.\")\n"
"\n"
"\n"
"    for id in x_ids:\n"
"        c = tplot_mod.Curve(y_var, id)\n"
"        c.set_x_variable(x_var)\n"
"        state.tplot.add_curve(c)\n"
"    state.tplot.print_show()\n"
"\n"
"\n"
"\n"
"def detour():\n"
"    \"\"\"\n"
"    Start a subprogram to plot deformed meshes.\n"
"    \"\"\"\n"
"    print \" DETOUR - a deformed mesh and contour plot program\"\n"
"    print\n"
"    state.subProgram = \"detour\"\n"
"    paraview.simple.SetActiveView(state.renderview)\n"
"    wireframe()\n"
"\n"
"def tplot():\n"
"    \"\"\"Start TPLOT subprogram to plot curves of variables over times.  Variables\n"
"    can be nodal, elemental, or global.\"\"\"\n"
"    print \" TPLOT - a time history or X-Y plot program\"\n"
"    print\n"
"    state.subProgram = \"tplot\"\n"
"    state.tplot = tplot_mod.TPlot()\n"
"    paraview.simple.SetActiveView(state.tplot.view)\n"
"\n"
"@_subprogram(\"tplot\")\n"
"def overlay(*args):\n"
"    \"\"\"Turn overlay on or off.\"\"\"\n"
"    value = True\n"
"    if args: value = _token_to_bool(args[0])\n"
"    state.tplot.overlay = value\n"
"    print \" Overlay curves for all variables\",\n"
"    if value: print \"on\"\n"
"    else: print \"off\"\n"
"    print\n"
"\n"
"@_subprogram(\"detour\")\n"
"def color(ncol=None):\n"
"    \"\"\"\n"
"    Set the maximum number of standard color scale colors to use on a color\n"
"    plot to ncol.\n"
"    \"\"\"\n"
"    maxColors = len(_standard_colors)\n"
"    if not ncol:\n"
"        print \"Set to\", state.numBlockColors, \"of\", maxColors, \"colors.\"\n"
"        return\n"
"\n"
"    ncol = int(ncol)\n"
"    if ncol > maxColors:\n"
"        print \"Setting to maximum of\", maxColors, \"instead of\", ncol\n"
"        ncol = maxColors\n"
"\n"
"    state.numBlockColors = ncol\n"
"    state.rebuildBlockLookupTable()\n"
"    _finish_plot_change()\n"
"\n"
"@_subprogram(\"detour\")\n"
"def spectrum(ncol=5):\n"
"    \"\"\"Set the number of contours to include in a paint or contour plot.\"\"\"\n"
"    state.numSpectrumColors = int(ncol)\n"
"    state.rebuildSpectrumLookupTable()\n"
"\n"
"    # Update the isosurfaces in the contour filter.\n"
"    if (state.hasPipelineObject('contour')):\n"
"        contour = state.contour\n"
"        contour.Isosurfaces = state.spectrumScalarValues\n"
"\n"
"    _finish_plot_change()\n"
"\n"
"    print \"Contour locations\", state.spectrumScalarValues\n"
"\n"
"def _updateMeshRender():\n"
"    \"\"\"Sets up the rendering of mesh based on the mlines and msurface state\n"
"    flags.  Returns the pipeline object that is being drawn so that further\n"
"    parameters (such as coloring) may be set up.\"\"\"\n"
"    todraw = state.reader\n"
"    if state.mlines:\n"
"        paraview.simple.Show(todraw)\n"
"        if state.msurface:\n"
"            paraview.simple.SetDisplayProperties(\n"
"                                     todraw, Representation=\"Surface With Edges\")\n"
"        else:\n"
"            paraview.simple.SetDisplayProperties(todraw,\n"
"                                                 Representation=\"Wireframe\")\n"
"    else:\n"
"        if state.msurface:\n"
"            paraview.simple.Show(todraw)\n"
"            paraview.simple.SetDisplayProperties(todraw,\n"
"                                                 Representation=\"Surface\")\n"
"        else:\n"
"            paraview.simple.Hide(todraw)\n"
"    return todraw\n"
"\n"
"@_subprogram(\"detour\")\n"
"def mlines(*args):\n"
"    \"\"\"\n"
"    Turns on/off the grid lines for block elements.  The default value\n"
"    is on.  If it is on, then the wireframe grid is drawn.  If it is off,\n"
"    the wireframe grid is not drawn.\n"
"    \"\"\"\n"
"    value = True\n"
"    if args: value = _token_to_bool(args[0])\n"
"    state.mlines = value\n"
"    print \" Display element mesh lines\",\n"
"    if value: print \"on\"\n"
"    else: print \"off\"\n"
"    print\n"
"    _updateMeshRender()\n"
"    _finish_plot_change()\n"
"\n"
"\n"
"def diskwrite(*args):\n"
"    \"\"\"\n"
"    Turns on/off writing to disk.  During non-interactive mode\n"
"    disk write default to on, else it is off.\n"
"    \"\"\"\n"
"    value = True\n"
"    if args: value = _token_to_bool(args[0])\n"
"    state.diskwrite = value\n"
"    print \" Write to disk\",\n"
"    if value: print \"on\"\n"
"    else: print \"off\"\n"
"    print\n"
"\n"
"@_subprogram(\"detour\")\n"
"def wireframe(*args):\n"
"    \"Sets the view to wireframe mesh mode, which displays the mesh lines.\"\n"
"    state.resetPipeline()\n"
"    state.mlines = 1\n"
"    state.msurface = 0\n"
"    todraw = _updateMeshRender()\n"
"    paraview.simple.SetDisplayProperties(todraw,\n"
"                                         ColorAttributeType=state._block_id_field,\n"
"                                         ColorArrayName=state._block_id_variable,\n"
"                                         LookupTable=state.blockLookupTable)\n"
"    _finish_plot_change()\n"
"\n"
"@_subprogram(\"detour\")\n"
"def solid():\n"
"    \"\"\"\n"
"    Sets the view to solid mesh mode, which paints each element using a\n"
"    different color for each element block.\n"
"    \"\"\"\n"
"    print \" Solid mesh plot\"\n"
"    print\n"
"    state.resetPipeline()\n"
"    state.msurface = 1\n"
"    todraw = _updateMeshRender()\n"
"    paraview.simple.SetDisplayProperties(todraw,\n"
"                                         ColorAttributeType=state._block_id_field,\n"
"                                         ColorArrayName=state._block_id_variable,\n"
"                                         LookupTable=state.blockLookupTable)\n"
"    _finish_plot_change()\n"
"\n"
"@_subprogram(\"detour\")\n"
"def paint(variable=None):\n"
"    \"\"\"Sets the view to paint contour mode, which paints contours of the\n"
"    given variable on the mesh.  The variable may be either nodal or element.\n"
"    If no variable is specified, the last selected variable is used.\"\"\"\n"
"    if variable:\n"
"        state.loadVariable(variable)\n"
"    if not state.currentVariable:\n"
"        raise BlotishError(\"No variable selected.\")\n"
"    if state.currentVariableType not in [NODE_VARIABLE, ELEMENT_VARIABLE]:\n"
"        raise BlotishError(\"Contour variable must be nodal or element variable\")\n"
"\n"
"    state.resetPipeline()\n"
"    state.mlines = 0\n"
"    state.msurface = 1\n"
"    todraw = _updateMeshRender()\n"
"    paraview.simple.SetDisplayProperties(todraw,\n"
"                                         ColorAttributeType=state.currentVariableType,\n"
"                                         ColorArrayName=state.currentVariable,\n"
"                                         LookupTable=state.spectrumLookupTable)\n"
"    _finish_plot_change()\n"
"\n"
"@_subprogram(\"detour\")\n"
"def contour(variable=None):\n"
"    \"\"\"Sets the view to contour mode, which plots the line or surface contours\n"
"    where the given variable is equal to one of the contour values (set with\n"
"    the spectrum command).\"\"\"\n"
"    if variable:\n"
"        state.loadVariable(variable)\n"
"    if state.currentVariableType != NODE_VARIABLE:\n"
"        raise BlotishError(\"Only nodal variables supported by contour right now.\")\n"
"    info = state.currentVariableInfo\n"
"    if (info.GetNumberOfComponents() > 1):\n"
"        raise BlotishError(state.currentVariable + \" is not a scalar.\")\n"
"\n"
"    state.resetPipeline()\n"
"    state.mlines = 0\n"
"    state.msurface = 0\n"
"    _updateMeshRender()\n"
"    contour = state.contour\n"
"    contour.ContourBy = state.currentVariable\n"
"    contour.Isosurfaces = state.spectrumScalarValues\n"
"    paraview.simple.Show(contour)\n"
"    paraview.simple.SetDisplayProperties(contour,\n"
"                                         Representation=\"Surface\",\n"
"                                         ColorAttributeType=state.currentVariableType,\n"
"                                         ColorArrayName=state.currentVariable,\n"
"                                         LookupTable=state.spectrumLookupTable)\n"
"    _finish_plot_change()\n"
"\n"
"@_subprogram(\"detour\")\n"
"def vector(variable=None):\n"
"    \"\"\"\n"
"    Sets the view to vector mode.  Pass the name of the variable as the\n"
"    argument, or pass nothing to use the last used variable.\n"
"    \"\"\"\n"
"    if variable:\n"
"        state.loadVariable(variable)\n"
"    if state.currentVariableType != 'POINT_DATA':\n"
"        raise BlotishError(\"Only nodal variables supported by vector right now.\")\n"
"    info = state.currentVariableInfo\n"
"    if (info.GetNumberOfComponents() < 2):\n"
"        raise BlotishError(state.currentVariable + \" is not a vector.\")\n"
"    glyph = state.glyph\n"
"    glyph.Vectors = state.currentVariable\n"
"    glyph.Orient = 1\n"
"    glyph.ScaleMode = 'vector'\n"
"\n"
"    #Determine scaling factor for vector.\n"
"    bounds = glyph[0].GetDataInformation().GetBounds()\n"
"    scalefactor = max(bounds[1]-bounds[0], bounds[3]-bounds[2],\n"
"                      bounds[5]-bounds[4])\n"
"    scalefactor = scalefactor/20\n"
"    divisor = 0\n"
"    for i in xrange(info.GetNumberOfComponents()):\n"
;

// From file /home/users/builder/rpm/BUILD/ParaView-3.12.0/Plugins/pvblot/blotish.py
static const char* module_blotish_blotish_string2 =
"        range = info.GetRange(i)\n"
"        divisor = max(divisor, abs(range[0]), abs(range[1]))\n"
"    if divisor > 1:\n"
"        scalefactor = scalefactor/divisor\n"
"    glyph.SetScaleFactor = scalefactor\n"
"\n"
"    state.resetPipeline()\n"
"    # Show the glyphs.\n"
"    paraview.simple.Show(glyph)\n"
"    # Show a transparent surface.\n"
"    state.mlines = 0\n"
"    state.msurface = 1\n"
"    surface = _updateMeshRender()\n"
"    paraview.simple.SetDisplayProperties(surface,\n"
"                                         ColorAttributeType=state._block_id_field,\n"
"                                         ColorArrayName=state._block_id_variable,\n"
"                                         LookupTable=state.blockLookupTable,\n"
"                                         Opacity=0.5)\n"
"    _finish_plot_change()\n"
"\n"
"\n"
"\n"
"\n"
"@async_io_helper.wrap(_get_io_helper)\n"
"def _tplot_plot(io_helper=None):\n"
"    if state.tplot.overlay:\n"
"        state.tplot.plot()\n"
"\n"
"        if state.diskwrite:\n"
"            fname = state.get_next_screenshot_filename(\"tplot_overlay\")\n"
"            paraview.simple.WriteImage(fname, state.tplot.view)\n"
"\n"
"        return\n"
"\n"
"    interactive = state.interactive\n"
"    nCurves = state.tplot.get_number_of_curves()\n"
"    for i in xrange(nCurves):\n"
"        state.tplot.plot(i)\n"
"\n"
"        if state.diskwrite:\n"
"            fname = state.get_next_screenshot_filename(\"tplot_curve%02d\"%i)\n"
"            paraview.simple.WriteImage(fname, state.tplot.view)\n"
"\n"
"        if interactive and i < nCurves-1:\n"
"            yield \"Enter 'Q' to quit, '' to continue. \"\n"
"            result = io_helper.get_input().lower()\n"
"            if result.startswith('q'):\n"
"                break\n"
"\n"
"\n"
"def _tplot_reset():\n"
"    state.tplot.reset()\n"
"    \n"
"def _detour_reset():\n"
"    wireframe()\n"
"\n"
"@_subprogram([\"detour\", \"tplot\"])\n"
"def plot(*args):\n"
"    \"Generates the current plot.\"\n"
"    if state.subProgram == \"tplot\": return _tplot_plot()\n"
"    if state.subProgram == \"detour\": return _detour_plot()\n"
"\n"
"@_subprogram([\"detour\", \"tplot\"])\n"
"def reset(*args):\n"
"    \"Reset the plot settings\"\n"
"    if state.subProgram == \"tplot\": return _tplot_reset()\n"
"    if state.subProgram == \"detour\": return _detour_reset()\n"
"\n"
"@_subprogram([\"detour\", \"tplot\"])\n"
"def times(*args):\n"
"    \"\"\"Select time steps\"\"\"\n"
"    try: state.time_selection.parse_times(\" \".join(args))\n"
"    except timestep_selection.Error, err: raise BlotishError(err)\n"
"    print \" Select specified whole times\"\n"
"    print \"    Number of selected times = %d\" % len(state.time_selection.selected_indices)\n"
"    print\n"
"\n"
"@_subprogram([\"detour\", \"tplot\"])\n"
"def steps(*args):\n"
"    \"\"\"Select time steps\"\"\"\n"
"    try: state.time_selection.parse_steps(\" \".join(args))\n"
"    except timestep_selection.Error, err: raise BlotishError(err)\n"
"    print \" Select specified whole times\"\n"
"    print \"    Number of selected times = %d\" % len(state.time_selection.selected_indices)\n"
"    print\n"
"\n"
"\n"
"@_subprogram(\"tplot\")\n"
"def xscale(min_value=None, max_value=None):\n"
"    \"\"\"Set the minimum and maximum values of the X axis\"\"\"\n"
"    state.tplot.set_xscale(_maybe_convert(min_value, float),\n"
"                           _maybe_convert(max_value, float))\n"
"\n"
"@_subprogram(\"tplot\")\n"
"def yscale(min_value=None, max_value=None):\n"
"    \"\"\"Set the minimum and maximum values of the X axis\"\"\"\n"
"    state.tplot.set_yscale(_maybe_convert(min_value, float),\n"
"                           _maybe_convert(max_value, float))\n"
"\n"
"@_subprogram(\"tplot\")\n"
"def xlabel(*args):\n"
"    \"\"\"Set the X axis label\"\"\"\n"
"    label = \" \".join(args)\n"
"    state.tplot.set_xlabel(label)\n"
"\n"
"@_subprogram(\"tplot\")\n"
"def ylabel(*args):\n"
"    \"\"\"Set the Y axis label\"\"\"\n"
"    label = \" \".join(args)\n"
"    state.tplot.set_ylabel(label)\n"
"\n"
"@_subprogram()\n"
"def nintv(value=None):\n"
"    \"\"\"Set nintv\"\"\"\n"
"    state.time_selection.set_nintv(_maybe_convert(value, int))\n"
"    state.time_selection.print_show()\n"
"\n"
"@_subprogram()\n"
"def zintv(value=None):\n"
"    \"\"\"Set zintv\"\"\"\n"
"    state.time_selection.set_zintv(_maybe_convert(value, int))\n"
"    state.time_selection.print_show()\n"
"\n"
"@_subprogram()\n"
"def deltime(value=None):\n"
"    \"\"\"Set deltime\"\"\"\n"
"    state.time_selection.set_deltime(_maybe_convert(value, float))\n"
"    state.time_selection.print_show()\n"
"\n"
"@_subprogram()\n"
"def tmin(value=None):\n"
"    \"\"\"Set deltime\"\"\"\n"
"    state.time_selection.set_tmin(_maybe_convert(value, float))\n"
"    state.time_selection.print_show()\n"
"\n"
"@_subprogram()\n"
"def tmax(value=None):\n"
"    \"\"\"Set deltime\"\"\"\n"
"    state.time_selection.set_tmax(_maybe_convert(value, float))\n"
"    state.time_selection.print_show()\n"
"\n"
"@_subprogram()\n"
"def alltimes():\n"
"    \"\"\"Set time selection mode to ALLTIMES\"\"\"\n"
"    state.time_selection.set_alltimes()\n"
"    state.time_selection.print_show()\n"
"\n"
"def show(name=\"\"):\n"
"    \"\"\"Show the given parameter name\"\"\"\n"
"    \n"
"    name = name.lower()\n"
"    if name in [\"tmin\", \"tmax\", \"nintv\", \"zintv\", \"times\", \"steps\"]:\n"
"        sel = state.time_selection\n"
"        count = 0\n"
"        state.time_selection.print_show()\n"
"        print \" %d selected time steps\" % len(sel.selected_times)\n"
"        for idx, time in zip(sel.selected_indices, sel.selected_times):\n"
"            print \"     %d)  (step %3d)  %f\" % (count+1, idx+1, time)\n"
"            count += 1\n"
"        print\n"
"\n"
"\n"
"@async_io_helper.wrap(_get_io_helper)\n"
"def _detour_plot(io_helper=None):\n"
"\n"
"    interactive = state.interactive\n"
"    plotTimes = state.time_selection.selected_times\n"
"    nSteps = len(plotTimes)\n"
"    view = state.renderview\n"
"    for stepIdx in xrange(nSteps):\n"
"        t = plotTimes[stepIdx]\n"
"        view.ViewTime = t\n"
"        paraview.simple.Render(view)\n"
"\n"
"        if state.diskwrite:\n"
"            paraview.simple.WriteImage(state.get_next_screenshot_filename(\"detour\"), view)\n"
"\n"
"        if interactive and stepIdx != nSteps-1:\n"
"            yield (\"Time %f: Enter 'C' to complete, 'Q' to quit, '' to continue. \" % t)\n"
"            result = io_helper.get_input().lower()\n"
"            if result.startswith('q'):\n"
"                break\n"
"            elif result.startswith('c'):\n"
"                interactive = False\n"
"\n"
"def autoplot(flag):\n"
"    \"\"\"    \n"
"    Turn autoplot on or off.  The flag should be 0 (off) or 1 (on).  When\n"
"    autoplot is on, then a command that changes the state of the plot\n"
"    automatically renders the last time step with the new features.\n"
"    \"\"\"\n"
"    if isinstance(flag, str):\n"
"        flag = int(flag)\n"
"    if (flag):\n"
"        state.autoplot = True\n"
"    else:\n"
"        state.autoplot = False\n"
"\n"
"x = 'x'\n"
"y = 'y'\n"
"z = 'z'\n"
"elevation = 'elevation'\n"
"azimuth = 'azimuth'\n"
"roll = 'roll'\n"
"\n"
"@_subprogram(\"detour\")\n"
"def rotate(*rotations):\n"
"    \"\"\"\n"
"    Rotates the 3D mesh.  Each (axis, ndeg) parameter pair specifies an\n"
"    axis of rotation (x, y, or z) or (elevation, azimuth, or roll) and the\n"
"    number of degrees to rotate.  The axes have the following meanings:\n"
"\n"
"      * x or elevation: rotate the camera around the focal point in the\n"
"        horizontal direction.\n"
"\n"
"      * y or azimuth: rotate the camera around the focal point in the vertical\n"
"        direction.\n"
"\n"
"      * z or roll: roll the camera about the view direction.\n"
"\n"
"    The identifiers elevation, azimuth, and roll can be abbreviated with any\n"
"    unique prefix.\n"
"    \"\"\"\n"
"    camera = paraview.simple.GetActiveCamera()\n"
"    # Some manging to rotation into list of pairs.\n"
"    rpairs = zip(rotations[::2], rotations[1::2])\n"
"    for axis, degrees in rpairs:\n"
"        axis = axis.lower()\n"
"        degrees = float(degrees)\n"
"        if axis == x or elevation.startswith(axis):\n"
"            sign = degrees/abs(degrees)\n"
"            degrees = abs(degrees)\n"
"            while degrees > 0:\n"
"                d = min(45, degrees)\n"
"                degrees = degrees - d\n"
"                camera.Elevation(sign*d)\n"
"                camera.OrthogonalizeViewUp()\n"
"        elif axis == y or azimuth.startswith(axis):\n"
"            camera.Azimuth(-degrees)\n"
"        elif axis == z or roll.startswith(axis):\n"
"            camera.Roll(degrees)\n"
"        else:\n"
"            print \"Unknown axis: \", axis\n"
"    _finish_plot_change()\n"
"\n"
"\n"
"@_subprogram(\"detour\")\n"
"def zoom(factor):\n"
"    \"\"\"\n"
"    Zooms the view by the given factor.  Factors bigger than 1 make the\n"
"    objects larger (e.g. 2 makes it look twice as big) and factors smaller\n"
"    than 1 make the objects look smaller (e.g. 0.5 makes it look twice as\n"
"    small).\n"
"\n"
"    If factor is one of the special identifiers reset, each, or mesh then\n"
"    the view is reset to fit the geometry in the view.\n"
"\n"
"    Note that this zoom is specified differently than the original blot\n"
"    program.  Also, the translat parameter is not supported.  Use the\n"
"    translate command instead.\n"
"    \"\"\"\n"
"    camera = paraview.simple.GetActiveCamera()\n"
"    if isinstance(factor, str):\n"
"        factor = factor.lower()\n"
"        if factor in [\"each\", \"mesh\", \"reset\"]:\n"
"            paraview.simple.ResetCamera()\n"
"            _finish_plot_change()\n"
"            return\n"
"        factor = float(factor)\n"
"    camera.Dolly(factor)\n"
"    _finish_plot_change()\n"
"\n"
"@_subprogram(\"detour\")\n"
"def translate(x, y):\n"
"    \"\"\"\n"
"    Translate the view in the x and y directions.  The x argument pans\n"
"    horizontally, the y vertically.  The values are normalized so that 1.0\n"
"    corresponds to (roughly) one screen width.\n"
"    \"\"\"\n"
"    x = float(x)\n"
"    y = float(y)\n"
"    from paraview.vtk import vtkMath\n"
"    camera = paraview.simple.GetActiveCamera()\n"
"    # Get basis vectors for the x and y translation\n"
"    xbasis = [0, 0, 0]\n"
"    ybasis = [0, 0, 0]\n"
"    vtkMath.Cross(camera.GetViewUp(), camera.GetDirectionOfProjection(), xbasis)\n"
"    vtkMath.Cross(xbasis, camera.GetDirectionOfProjection(), ybasis)\n"
"    vtkMath.Normalize(xbasis)\n"
"    vtkMath.Normalize(ybasis)\n"
"    # Scale the basis functions by the size of the viewable region.\n"
"    cameradist = 0\n"
"    for fpv, posv in zip(camera.GetFocalPoint(), camera.GetPosition()):\n"
"        cameradist = cameradist + (fpv-posv)*(fpv-posv)\n"
"    cameradist = math.sqrt(cameradist)\n"
"    height = cameradist*math.tan(math.radians(camera.GetViewAngle()))\n"
"    xbasis = map(lambda a: a*height, xbasis)\n"
"    ybasis = map(lambda a: a*height, ybasis)\n"
"    # Compute the translation vector\n"
"    translation = map(lambda xb, yb: xb*x + yb*y, xbasis, ybasis)\n"
"    # Translate focal point and position\n"
"    fp = camera.GetFocalPoint()\n"
"    fp = map(lambda a, b: a + b, fp, translation)\n"
"    camera.SetFocalPoint(fp)\n"
"    pos = camera.GetFocalPoint()\n"
"    pos = map(lambda a, b: a + b, pos, translation)\n"
"    camera.SetFocalPoint(pos)\n"
"\n"
"    _finish_plot_change()\n"
"\n"
"def _token_to_bool(token):\n"
"    token = str(token).lower()\n"
"    if token in [\"1\", \"on\", \"true\", \"yes\"]:\n"
"        return True\n"
"    if token in [\"0\", \"off\", \"false\", \"no\"]:\n"
"        return False\n"
"    raise BlotishError(\"Expected \\\"ON\\\" or \\\"OFF\\\"\")\n"
"\n"
"\n"
"def _maybe_convert(token, number_class):\n"
"    \"\"\"Attempts to convert the argument to the given number class if the\n"
"    argument is not None.  Raises an exception on a conversion error.  If the\n"
"    argument is None, returns None without raising any error.\"\"\"\n"
"    if token is None: return None\n"
"    try: num = number_list_parser.convert_to_number_class(token, number_class)\n"
"    except number_list_parser.Error, err: raise BlotishError(err)\n"
"    return num\n"
"\n"
"\n"
"\n";
// Get single string
char* module_blotish_blotish_source()
{
  size_t len = ( 0
    + strlen(module_blotish_blotish_string0)
    + strlen(module_blotish_blotish_string1)
    + strlen(module_blotish_blotish_string2) );
  char* res = new char[ len + 1];
  res[0] = 0;
  strcat(res, module_blotish_blotish_string0);
  strcat(res, module_blotish_blotish_string1);
  strcat(res, module_blotish_blotish_string2);
  return res;
}



#endif
