// Loadable modules
//
// Generated by ../../bin/kwProcessXML
//
#ifndef __module_pvblot__h
#define __module_pvblot__h

#include <string.h>


// From file /home/users/builder/rpm/BUILD/ParaView-3.12.0/Plugins/pvblot/pvblot.py
static const char* module_pvblot_pvblot_string0 =
"r\"\"\"\n"
"\n"
"An interpreter that provides blot-like syntax and functionality.  To run\n"
"the interpreter simply call the start function.\n"
"\n"
"\"\"\"\n"
"\n"
"#==============================================================================\n"
"#\n"
"#  Program:   ParaView\n"
"#  Module:    pvblot.py\n"
"#\n"
"#  Copyright (c) Kitware, Inc.\n"
"#  All rights reserved.\n"
"#  See Copyright.txt or http://www.paraview.org/HTML/Copyright.html for details.\n"
"#\n"
"#     This software is distributed WITHOUT ANY WARRANTY; without even\n"
"#     the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR\n"
"#     PURPOSE.  See the above copyright notice for more information.\n"
"#\n"
"#==============================================================================\n"
"\n"
"#-------------------------------------------------------------------------\n"
"# Copyright 2009 Sandia Corporation.\n"
"# Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,\n"
"# the U.S. Government retains certain rights in this software.\n"
"#-------------------------------------------------------------------------\n"
"\n"
"import cmd\n"
"import inspect\n"
"import blotish\n"
"import blot_common\n"
"import traceback\n"
"import sys\n"
"\n"
"class _PVBlotInterp(cmd.Cmd):\n"
"    \"\"\"pvblot - A blot-like interpreter using ParaView as a back end.\n"
"\n"
"    The pvblot interpreter reads commands in a manner similar to the SEACAS\n"
"    blot command and performs visualization using the ParaView framework.\n"
"    The images you get from pvblot will often provide more information than\n"
"    those generated by the original blot tool.  Comments begin with \"$\" and\n"
"    continue to the end of the line.  Commands are case insensitive and can\n"
"    be abbreviated with any unique prefix.  Keep in mind that there are\n"
"    some differences in the blot commands and those offered here.  There\n"
"    are also many blot commands that are not replicated here.\n"
"    \"\"\"\n"
"    prompt = \"BLOT> \"\n"
"    _blotish_commands = {}\n"
"    def __init__(self, completekey='tab', stdin=None, stdout=None):\n"
"        cmd.Cmd.__init__(self, completekey, stdin, stdout)\n"
"        for funcname, func in blotish.__dict__.iteritems():\n"
"            if funcname[0] == '_' or not inspect.isfunction(func): continue\n"
"            if not func.__doc__ or not len(func.__doc__):\n"
"                raise blotish.BlotishError(\"Method %s is undocumented.\" % funcname)\n"
"            self._blotish_commands[funcname] = func\n"
"        # Insert dummy commands for help.\n"
"        self._blotish_commands['exit'] = _PVBlotInterp.do_exit\n"
"        self._error_flag = False\n"
"\n"
"    def get_unique_command(self, command_name):\n"
"        \"\"\"Given a command name, returns the function that implements it.\n"
"        The command name should be in all lower case.  The command name\n"
"        need only cover the first part of the command uniquely.  If no\n"
"        command is found, or more than one command is found, raises a\n"
"        BlotishError.\"\"\"\n"
"        valid_commands = []\n"
"        for existing_command in self._blotish_commands.keys():\n"
"            if existing_command.startswith(command_name):\n"
"                valid_commands.append(existing_command)\n"
"        if not valid_commands:\n"
"            # If no command was found, maybe the command name is a variable name\n"
"            func = blotish._find_variable_command(command_name)\n"
"            if func is not None:\n"
"                return func\n"
"            else:\n"
"                raise blotish.BlotishError(\"No such command '%s'\" % command_name)\n"
"        if len(valid_commands) > 1:\n"
"            raise blotish.BlotishError(\n"
"                  \"Command not found.  Possible commands: %s\" % str(\", \").join(valid_commands))\n"
"        return self._blotish_commands[valid_commands[0]]\n"
"\n"
"    def remove_comments_from_line(self, line):\n"
"        \"\"\"Searches a command string for the comment character.  If found,\n"
"        removes the comment character and everything following it to create\n"
"        a new command string.  Returns the new command string converted to\n"
"        lowercase.\"\"\"\n"
"        comment_start = line.find('$')\n"
"        if (comment_start >= 0):\n"
"            line = line[:comment_start]\n"
"        return line.lower()\n"
"\n"
"    def precmd(self, command):\n"
"        \"\"\"This method is called to preprocess a command input.\"\"\"\n"
"        return self.remove_comments_from_line(command)\n"
"\n"
"    def do_help(self, command_name):\n"
"        \"\"\"The given argument is a string representing a command name.  The\n"
"        string may be empty.  Prints documentation for the command name if given\n"
"        else prints a list of available command names.\"\"\"\n"
"        if not command_name:\n"
"            print _PVBlotInterp.__doc__\n"
"            print \"The following commands are supported:\"\n"
"            print \"  \",\n"
"            blotish_commands = self._blotish_commands.keys()\n"
"            blotish_commands.sort()\n"
"            for c in blotish_commands:\n"
"                print c,\n"
"            print\n"
"            print\n"
"            print \"For more information on any command, try help <command>.\"\n"
"            return\n"
"        try:\n"
"            command = self.get_unique_command(command_name)\n"
"            print command.__doc__\n"
"        except blotish.BlotishError, err:\n"
"            blot_common.print_blot_error(err)\n"
"\n"
"    def do_exit(self, args):\n"
"        \"Exit the blot interpreter.\"\n"
"        return True\n"
"\n"
"    def do_eof(self, line):\n"
"        \"\"\"Print a blank line and return True to signal we are ready to exit.\"\"\"\n"
"        print \"\"\n"
"        return True\n"
"\n"
"    def handle_exception(self, exception):\n"
"        \"\"\"Print a formatted error message if the exception is a BlotishError\n"
"        else print a traceback.  Does not cause the interpreter to quit.\"\"\"\n"
"        self._error_flag = True\n"
"        if exception.__class__ is blotish.BlotishError:\n"
"            blot_common.print_blot_error(exception)\n"
"        else:\n"
"            traceback.print_exc(file=sys.stdout)\n"
"\n"
"    def handle_async_input(self, s):\n"
"        \"\"\"Takes an input string.  If there is an object waiting for asynchronous\n"
"        input that uses the string then return true, if an exception is thrown then\n"
"        it prints the exception and possibly a traceback and returns true.  If the\n"
"        input is not used at all then return false.\"\"\"\n"
"        ret = True\n"
"        try: ret = blotish._handle_input(s)\n"
"        except Exception, err:\n"
"            self.handle_exception(err)\n"
"\n"
"        # Update the prompt, some commands cause the prompt to change\n"
"        self.prompt = blotish._get_prompt()\n"
"        return ret\n"
"      \n"
"    def emptyline(self):\n"
"        \"\"\"If an object is waiting for asynchronous input pass it the empty string\"\"\"\n"
"        self.handle_async_input(\"\")\n"
"\n"
"    def default(self, s):\n"
"        \"\"\"This method is called to handle all blotish commands.\"\"\"\n"
"\n"
"        # First see if the asynchronous IO helper eats the string\n"
"        if self.handle_async_input(s):\n"
"            return False\n"
"\n"
"        # Split the string into tokens.  Treat the first token as the command\n"
"        # name, look for a blotish command and call it with the remaining tokens.\n"
"        args = s.split()\n"
"        try:\n"
"            command = self.get_unique_command(args[0])\n"
"            command(*args[1:])\n"
"        except Exception, err:\n"
"            self.handle_exception(err)\n"
"\n"
"        # Update the prompt, some commands cause the prompt to change\n"
"        self.prompt = blotish._get_prompt()\n"
"\n"
"\n"
"\n"
"def initialize(filename=None):\n"
"    \"\"\"Initialize the pvblot interpreter for non-interactive use.\n"
"\n"
"    Normally you just call start and it takes care of reading and executing\n"
"    commands.  However, in situation where you are not reading from stdin and\n"
"    stdout, you may need some other controlling mechanism to call methods\n"
"    one at a time.  In that case, use the initialize, execute, and finalize\n"
"    functions.\n"
"    \"\"\"\n"
"    # Turn off progress printing while running\n"
"    import paraview.servermanager\n"
"    global wasProgressPrinting\n"
"    wasProgressPrinting = paraview.servermanager.GetProgressPrintingIsEnabled()\n"
"    paraview.servermanager.SetProgressPrintingEnabled(False)\n"
"\n"
"    blotish._init_blotish(filename)\n"
"    global interpreter\n"
"    interpreter = _PVBlotInterp()\n"
"\n"
"def execute(command):\n"
"    \"\"\"Run a command in the interpreter.\n"
"\n"
"    Normally you just call start and it takes care of reading and executing\n"
"    commands.  However, in situation where you are not reading from stdin and\n"
"    stdout, you may need some other controlling mechanism to call methods\n"
"    one at a time.  In that case, use the initialize, execute, and finalize\n"
"    functions.\n"
"\n"
"    This function returns a true value if the exit command is given.\n"
"    \"\"\"\n"
"    global interpreter\n"
"    command = interpreter.precmd(command)\n"
"    stop = interpreter.onecmd(command)\n"
"    stop = interpreter.postcmd(stop, command)\n"
"    return stop\n"
"\n"
"def finalize():\n"
"    \"\"\"Clean up the interpreter.\n"
"\n"
"    Normally you just call start and it takes care of reading and executing\n"
"    commands.  However, in situation where you are not reading from stdin and\n"
"    stdout, you may need some other controlling mechanism to call methods\n"
"    one at a time.  In that case, use the initialize, execute, and finalize\n"
"    functions.\n"
"    \"\"\"\n"
"    global interpreter\n"
"    del interpreter\n"
"    blotish._cleanup()\n"
"\n"
"    # Set the progress printing state to whatever it was before\n"
"    import paraview.servermanager\n"
"    global wasProgressPrinting\n"
"    paraview.servermanager.SetProgressPrintingEnabled(wasProgressPrinting)\n"
"\n"
"def execute_file(filename):\n"
"    \"\"\"Executes each command in the file.  If an exit command is executed\n"
"    this method will stop execution and return True.  This method returns\n"
"    False after all commands have been executed.  If a command causes an\n"
"    error then execution stops but this method will still return False.\n"
"    This can only be called after calling initialize()\"\"\"\n"
"    try:\n"
"        f = open(filename, 'r')\n"
"    except IOError, err:\n"
"        print \"Could not open file\", filename\n"
"        return\n"
"    blotish._set_interactive(False)\n"
"    exit_flag = False\n"
"    for line in f:\n"
"        line = line.rstrip()\n"
"        exit_flag = execute(line)\n"
"        if exit_flag: break\n"
"        if interpreter._error_flag: break\n"
"    blotish._set_interactive(True)\n"
"    return exit_flag\n"
"        \n"
"\n"
"\n"
"def start(data_file, script_file=None):\n"
"    \"\"\"Start the pvblot interpreter.\"\"\"\n"
"\n"
"    # Try to start up the interpreter\n"
"    try:\n"
"        initialize(data_file)\n"
"    except blotish.BlotishError, err:\n"
"        blot_common.print_blot_error(err)\n"
"        return\n"
"\n"
"    # Maybe run a script\n"
"    exit_flag = False\n"
"    if script_file:\n"
"        exit_flag = execute_file(script_file)\n"
"\n"
"    # Start the interpreter unless the script called exit\n"
"    if not exit_flag:\n"
"        global interpreter\n"
"        interpreter.cmdloop()\n"
"\n"
"    # Cleanup\n"
"    finalize()\n"
"\n"
"\n"
"def main():\n"
"    \"\"\"\n"
"    Call start() after parsing sys.argv for arguments:\n"
"        [-input script_file] data_file\n"
"    \"\"\"\n"
"\n"
"    data_file = None\n"
"    script_file = None\n"
"    if sys.argv and len(sys.argv) > 1:\n"
"        if sys.argv[1] == \"-input\":\n"
"            try: script_file = sys.argv[2]\n"
"            except IndexError:\n"
"                print \"You need to specify a blot script for the -input option.\"\n"
"                return\n"
"            try: data_file = sys.argv[3]\n"
"            except IndexError:\n"
"                print \"You need to specify a data filename.\"\n"
"                return\n"
"        else: data_file = sys.argv[1]\n"
"    start(data_file, script_file)\n"
"\n"
"\n"
"#If we are running this source file directly, start it up.\n"
"if __name__ == '__main__':\n"
"    main()\n"
"\n"
"\n";
// Get single string
char* module_pvblot_pvblot_source()
{
  size_t len = ( 0
    + strlen(module_pvblot_pvblot_string0) );
  char* res = new char[ len + 1];
  res[0] = 0;
  strcat(res, module_pvblot_pvblot_string0);
  return res;
}



#endif
