/*=========================================================================

   Program: ParaView
   Module:    pqParaViewPlugin.cxx.in

   Copyright (c) 2005,2006 Sandia Corporation, Kitware Inc.
   All rights reserved.

   ParaView is a free software; you can redistribute it and/or modify it
   under the terms of the ParaView license version 1.2. 

   See License_v1.2.txt for the full ParaView license.
   A copy of this license can be obtained by contacting
   Kitware Inc.
   28 Corporate Drive
   Clifton Park, NY 12065
   USA

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE AUTHORS OR
CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

========================================================================*/
#include "pvblot_Plugin.h"

#include "vtkObjectFactory.h"

//-----------------------------------------------------------------------------
#ifdef plugin_type_servermanager
# ifdef INITIALIZE_WRAPPING
#  include "vtkClientServerInterpreterInitializer.h"
#  include "vtkClientServerInterpreter.h"

extern "C" void pvblot_Initialize(vtkClientServerInterpreter *);

extern "C" void VTK_EXPORT pvblot_CombinedInitialize(
  vtkClientServerInterpreter *interp)
{
  pvblot_Initialize(interp);

  // Now initialize any extra kits as requested.
  
}
# endif
#endif

//-----------------------------------------------------------------------------
#ifdef plugin_type_servermanager


#endif

//-----------------------------------------------------------------------------
#ifdef plugin_type_gui
#define PUSH_BACK_PV_INTERFACES(arg)\
  arg.push_back(new PVBlotPluginActionsImplementation(this));\



#include "PVBlotPluginActionsImplementation.h"
#endif

//-----------------------------------------------------------------------------
#ifdef plugin_type_servermanager
vtkClientServerInterpreterInitializer::InterpreterInitializationCallback
pvblot_Plugin::GetInitializeInterpreterCallback()
{
# ifdef INITIALIZE_WRAPPING
  return pvblot_CombinedInitialize;
# endif
  return NULL;
}

//-----------------------------------------------------------------------------
void pvblot_Plugin::GetXMLs
  (vtkstd::vector<vtkstd::string> &xmls)
{
# ifdef PUSH_BACK_XML_INTERFACES
  PUSH_BACK_XML_INTERFACES(xmls);
# endif
  (void)xmls;
}
#endif

//-----------------------------------------------------------------------------
#ifdef plugin_type_gui
QObjectList pvblot_Plugin::interfaces()
{
  QObjectList ifaces;
#ifdef PUSH_BACK_PV_INTERFACES
  PUSH_BACK_PV_INTERFACES(ifaces);
#endif
  return ifaces;
}
#endif

//-----------------------------------------------------------------------------
#ifdef plugin_type_python
#include "/home/users/builder/rpm/BUILD/ParaView-3.12.0/build/Plugins/pvblot/WrappedPython_pvblot_blotish.h"
#include "/home/users/builder/rpm/BUILD/ParaView-3.12.0/build/Plugins/pvblot/WrappedPython_pvblot_pvblot.h"
#include "/home/users/builder/rpm/BUILD/ParaView-3.12.0/build/Plugins/pvblot/WrappedPython_pvblot_blot_common.h"
#include "/home/users/builder/rpm/BUILD/ParaView-3.12.0/build/Plugins/pvblot/WrappedPython_pvblot_number_list_parser.h"
#include "/home/users/builder/rpm/BUILD/ParaView-3.12.0/build/Plugins/pvblot/WrappedPython_pvblot_tplot.h"
#include "/home/users/builder/rpm/BUILD/ParaView-3.12.0/build/Plugins/pvblot/WrappedPython_pvblot_async_io_helper.h"
#include "/home/users/builder/rpm/BUILD/ParaView-3.12.0/build/Plugins/pvblot/WrappedPython_pvblot_timestep_selection.h"


void pvblot_Plugin::GetPythonSourceList(vtkstd::vector<vtkstd::string>& modules,
  vtkstd::vector<vtkstd::string>& sources,
  vtkstd::vector<int> &package_flags)
{
  const char *moduleNames[] = {
    "blotish",
        "pvblot",
        "blot_common",
        "number_list_parser",
        "tplot",
        "async_io_helper",
        "timestep_selection"
  };
  char *moduleSources[] = {
    module_blotish_blotish_source(),
        module_pvblot_pvblot_source(),
        module_blot_common_blot_common_source(),
        module_number_list_parser_number_list_parser_source(),
        module_tplot_tplot_source(),
        module_async_io_helper_async_io_helper_source(),
        module_timestep_selection_timestep_selection_source()
  };
  const int packageFlags[] = {
    0, 0, 0, 0, 0, 0, 0
  };

  int num_modules = sizeof(moduleNames)/sizeof(const char *);
  for (int cc=0; cc < num_modules; cc++)
    {
    modules.push_back(moduleNames[cc]);
    sources.push_back(moduleSources[cc]);
    package_flags.push_back(packageFlags[cc]);

    // free allocated memory.
    delete moduleSources[cc];
    moduleSources[cc] = NULL;
    }
}
#endif

//-----------------------------------------------------------------------------
// Mark this as a ParaView-ServerManager plugin.
PV_PLUGIN_EXPORT(pvblot, pvblot_Plugin)

//-----------------------------------------------------------------------------
#ifdef BUILD_SHARED_LIBS
#ifdef plugin_type_gui

// Required to mark this as a Qt plugin.
Q_EXPORT_PLUGIN2(pvblot,  pvblot_Plugin)
#endif
#endif


